/************************************************************************
 *   IRC - Internet Relay Chat, m_extra/m_silence.c
 *   This file is copyright (C) 2001 Andrew Suffield
 *                                    <asuffield@users.sourceforge.net>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */

#include "config.h"
#include "client.h"
#include "msg.h"
#include "handlers.h"
#include "numeric.h"
#include "struct.h"
#include "irc_string.h"
#include "memory.h"
#include "list.h"
#include "send.h"
#include "ircd.h"
#include "channel.h"
#include "extra.h"

extern int m_silence(struct Client *, struct Client *, int, char **);

struct Message silence_msgtab = {
  "SILENCE", 0, 2, 0, MFLG_SLOW, 0,
  {m_unregistered, m_silence, m_silence},
  NULL, NULL
};

static int
del_silence(struct Client *sptr, char *mask)
{
  Link **lp, *tmp;

  for (lp = &(sptr->user->silence); *lp; lp = &((*lp)->next))
    if (irccmp(mask, (*lp)->value.cp) == 0)
      {
	tmp = *lp;
	*lp = tmp->next;
	MyFree(tmp->value.cp);
	free_link(tmp);
	return 0;
      }

  return -1;
}

static int
add_silence(struct Client *sptr, char *mask)
{
  Link *lp;
  int cnt = 0, len = 0;

  for (lp = sptr->user->silence; lp; lp = lp->next)
    {
      len += strlen(lp->value.cp);
      if (MyClient(sptr))
	{
	  if ((len > MAXSILELENGTH) || (++cnt >= MAXSILES))
	    {
	      sendto_one(sptr, form_str(ERR_SILELISTFULL), me.name,
			 sptr->name, mask);
	      return -1;
	    }
	  else
	    {
	      if (match(lp->value.cp, mask))
		return -1;
	    }
	}
      else if (!strcasecmp(lp->value.cp, mask))
	{
	  return -1;
	}
    }

  lp = make_link();
  lp->value.cp = (char *) MyMalloc(strlen(mask) + 1);
  (void) strcpy(lp->value.cp, mask);
  lp->next = sptr->user->silence;
  sptr->user->silence = lp;

  return 0;
}

/*
 * is_silence : Does the actual check whether sptr is allowed
 *              to send a message to acptr.
 *              Both must be registered persons.
 * If sptr is silenced by acptr, his message should not be propagated,
 * but moreover, if this is detected on a server not local to sptr
 * the SILENCE mask is sent upstream.
 */
int
_is_silenced(aClient * sptr, aClient * acptr)
{
  Link *lp;
  anUser *user;
  static char sender[HOSTLEN + NICKLEN + USERLEN + 5];

  if (!(acptr->user) || !(lp = acptr->user->silence) || !(user = sptr->user))
    return 0;
  ircsprintf(sender, "%s!%s@%s", sptr->name, sptr->username, sptr->host);
  for (; lp; lp = lp->next)
    {
      if (match(lp->value.cp, sender))
	{
	  if (!MyConnect(sptr))
	    {
	      sendto_one(sptr->from, ":%s SILENCE %s :%s", acptr->name,
			 sptr->name, lp->value.cp);
	      lp->flags = 1;
	    }
	  return 1;
	}
    }
  return 0;
}

/*
** m_silence - borrowed from Cyclone
**      parv[0] = sender prefix
**      parv[1] = mask (NULL sends the list)
*/
int
m_silence(aClient * cptr, aClient * sptr, int parc, char *parv[])
{
  Link *lp;
  aClient *acptr;
  char c = 0, *cp;

  acptr = sptr;
  if (parc < 2 || *parv[1] == '\0' || (acptr = find_person(parv[1], NULL)))
    {
      if (!(acptr->user))
	return 0;
      for (lp = acptr->user->silence; lp; lp = lp->next)
	sendto_one(sptr, form_str(RPL_SILELIST), me.name,
		   sptr->name, acptr->name, lp->value.cp);
      sendto_one(sptr, form_str(RPL_ENDOFSILELIST), me.name, acptr->name);
      return 0;
    }
  cp = parv[1];
  c = *cp;
  if (c == '-' || c == '+')
    cp++;
  else if (!(index(cp, '@') || index(cp, '.') ||
	     index(cp, '!') || index(cp, '*')))
    {
      sendto_one(sptr, form_str(ERR_NOSUCHNICK), me.name, parv[0], parv[1]);
      return -1;
    }
  else
    c = '+';
  cp = pretty_mask(cp);
  if ((c == '-' && !del_silence(sptr, cp)) ||
      (c != '-' && !add_silence(sptr, cp)))
    {
      sendto_serv_butone(NULL, ":%s SILENCE %c%s", sptr->name, c, cp);
    }
  return 0;
}
